#ifndef _SOCKETUDP_H_
#define _SOCKETUDP_H_

#include "socket.h"
#include "cqueue.h"
#include <vector>

typedef struct
{
  std::string ip_address;
  std::string new_data_avail_event_id;
  CQueue data_queue;
}TUDPClient;

class CSocketUDP : public CSocket
{
  private:
    int associated_port;
    std::vector<TUDPClient *> clients;
  protected:
    /**
     * \brief Connect function
     *
     * This function is called when the base class open() function is called, and 
     * it is used to perform the connection to a server. This function requires a 
     * TSocket_info structure as parameter which must be provided to the open() 
     * call. It also calls the hard_open() function of the CSocket class to 
     * initialize the base class attributes.
     *
     * The IP address and port within this structure are used to try to stablish
     * a communication with a remote server. If the server is already listening,
     * the function returns normally with all the internal attributes properly
     * initialized.
     *
     * If the server is not yet listening, this function throws a CSocketNoConnection
     * exception. This exception can be catched and used to iterate the process 
     * until the server is available or terminate the applcation. In case of any
     * other error, this function throws a CSocketException.
     *
     * After the connection is set, it is still necessary to call the config() 
     * function to properly configure the object to send and receive information.
     *
     * \param comm_dev a valid pointer to a TSocket_info structure with the IP
     *                 address and listening port of the desired server. See the
     *                 documentation on the TSocket_info structure for more
     *                 information on the IP address and port format.
     */
    virtual void hard_open(void *comm_dev=NULL);
    /**
     * \brief Function to actually read from the device
     *
     * This function is automatically called when the new data received event is 
     * activated. The read() function from the base class gets data from the 
     * internal queue, so this function is not used. It must try to read the 
     * ammount of data specified and store it in the data buffer provided without 
     * blocking. Also, it must return the number of bytes actually read from the 
     * devicve, since they may be different than the desired value.
     *
     * In case of any error, this function throws a CSocketException exception.
     *
     * \param data a reference to the buffer where the received data must be 
     * copied. The necessary memory for this buffer must be allocated before 
     * calling this function and have enough size to store all the desired data.
     * If this buffer is not initialized, the function throws an exception.
     *
     * \param len a positive interger that indicates the number of byte to be
     * read from the communication device. This value must be at most the length 
     * of the data buffer provided to the function.
     *
     * \return an integer with the number of bytes actually read. These number 
     * coincide with the desired number if there is enough data in the internal 
     * queue, but it could be smaller if not.
     */
    virtual int hard_read(unsigned char *data, int len);
    /**
     * \brief function to close the client socket
     *
     * This function is called when the close() function of the base class is 
     * called. It calls the hard_close() function of the CSocket class to
     * actually terminate the connection and change all the internal attributes 
     * to reflect that (the connection flag is set back to false and the server
     * IP address and port are set to the default values.
     *
     * In case of any error, this function throws a CSocketException.
     */
    virtual void hard_close(void);
  public:
    CSocketUDP(const std::string &sock_id);
    std::string add_client(std::string &ip);
    int write_to(std::string &ip,int port,unsigned char *data,int len);
    int read_from(std::string &ip, unsigned char *data, int len);
    int get_num_data_from(std::string &ip);
    ~CSocketUDP();
};

#endif
